/************************************************************************************************
 Author        : ADI - Apps            www.analog.com

 Date          : July 2009

 File          : SPI_Slave.c

 Hardware      : ADuC7122

 Description   : This SPI Slave example shows how to initialise the SPI peripheral in Slave mode.	
*************************************************************************************************/
// Bit Definitions
#define BIT0  0x01
#define BIT1  0x02
#define BIT2  0x04
#define BIT3  0x08
#define BIT4  0x10
#define BIT5  0x20
#define BIT6  0x40
#define BIT7  0x80
#define BIT8  0x100
#define BIT9  0x200
#define BIT10 0x400
#define BIT11 0x800
#define BIT12 0x1000
#define BIT13 0x2000
#define BIT14 0x4000
#define BIT15 0x8000
#define BIT16 0x10000
#define BIT17 0x20000
#define BIT19 0x80000
#define BIT20 0x100000
#define BIT24 0x1000000
#define BIT25 0x2000000
#define BIT28 0x10000000 

#include <Analogdevices\ioaduc7122.h>
#include <intrinsics.h>

unsigned char szStoreData[] = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x12, 0x55};	// Array to send to Slave
unsigned char ucStoreCount = 0;      	// Array index variable for szStoreData[]
unsigned char ucWaitingForXIRQ0 = 1; 	// Flag to begin write from Slave - controlled by XIRQ0
unsigned char ucReadOnly = 0;
unsigned char ucTxCnt = 0;
unsigned char ucbuffer;
unsigned char ucDummyRd=0;
int Underflow=0;
unsigned char ucTxCount = 0;
int main(void)
{
        POWKEY1 = 0x1;
	POWCON = 0x00;		                // Set core to max CPU speed of 10.24Mhz
	POWKEY2 = 0xF4;

        // Configure P0.2, P0.3,P0.4 and P0.5 for SPI mode
	GP0CON = BIT8 + BIT12 + BIT16 + BIT20;  // Select SPI alternative function 
        GP2DAT = BIT24;        		// Set P2.0 as an output
        __enable_interrupt();
        // Enable SPI Slave mode and interrupt sources
	SPICON = BIT0       	                // Enable SPI 
		   + BIT7                       // Clk pulses high at start of each bit 
		   + BIT9                       // Initiate transfer on write to Tx FIFO
		   + BIT11
                   + BIT8;		        //Continuous transfer mode + Enable IRQ on 1 byte transfer
	IRQEN = BIT14;// + BIT19; 		// Enable SPI + XIRQ1interrupt
        SPICON |= BIT12;                        // Flush Rx FIFO
	SPICON &= ~BIT12;
	ucStoreCount = 0;
	while (1)
	{
            if(ucReadOnly == 1 )
            {
            }
	}
}

 __irq void IRQ_Handler(void)
{
	unsigned long IRQSTATUS = 0;
	unsigned int SPIMSTATUS = 0;

	IRQSTATUS = IRQSTA;	  		// Read off IRQSTA register

	if ((IRQSTATUS & BIT14) == BIT14)	//If SPI Slave interrupt source
	{
	   SPIMSTATUS = SPISTA;
	   if ((SPIMSTATUS & BIT4) == BIT4) 	// If SPI Slave Tx Underflow
	   {
	        Underflow=1;
	   }

	   if ((SPIMSTATUS & BIT5) == BIT5) 	// If SPI Slave Tx IRQ
	   {            
             	if (ucTxCount < 6)	        // Have max 6 bytes been sent?				
		{
                    GP2SET = BIT16; 
	            SPITX = szStoreData[ucTxCount++];
                    GP2CLR = BIT16;
                }
           }
	   if ((SPIMSTATUS & BIT6) == BIT6)     // If SPI Slave Rx IRQ
	   {
		if (ucStoreCount < 6)		// Have max 6 bytes been received?
		{
		    if ( ucReadOnly == 0)
		    {
                        GP2SET = BIT16; 
			szStoreData[ucStoreCount] = SPIRX;  // Read Rx buffer
			GP2CLR = BIT16;
                        ucbuffer = szStoreData[ucStoreCount];
			ucStoreCount++;
			if (ucStoreCount >=6)
			{
			    ucStoreCount = 0;
			    ucReadOnly = 1;
                            SPICON |= BIT6;
                            ucTxCount=2;                        
                            SPICON |= BIT13;    // Flush Tx FIFO
                            SPICON &= ~BIT13;
                            SPITX = szStoreData[0];
                            SPITX = szStoreData[1];

   
			}
		    }
		    else  // Read from slave only
		    {
                            
                        if(ucTxCount < 6)
                        {
                            ucDummyRd = SPIRX;
                            // GP2SET = BIT16; 
			    // SPITX = szStoreData[ucTxCount++];  
                            // GP2CLR = BIT16;
                        }
		    }
		}
	   }
	   // Test for SPICON.8/SPISTA.7 - remove reading fo SPIRX register
	   if ((SPIMSTATUS & BIT7) == BIT7)     // If SPI Slave Rx FIFO Overflow status IRQ
	   {
              Underflow=2;
	   }
	}
	if ((IRQSTATUS & BIT19) == BIT19)       //XIRQ1 interrupt source
	{		
		ucWaitingForXIRQ0 = 0;		// Enable Write from Slave.
	}
}
